package ffp_examples;

import java.awt.AWTEvent;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.font.TextHitInfo;
import java.awt.im.InputMethodRequests;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;

import ffp.FFP;
import ffp.FFPAUX;
import ffp.FFPC;

abstract class DemoBase implements FFPC {

	static final int KEY_PRESSED = 401;
	static final int KEY_RELEASED = 402;

	static final int KEY_SHIFT = 0x10;
	static final int KEY_ESCAPE = 0x1b;
	static final int KEY_SPACE = 0x20;

	static final int KEY_LEFT = 0x25;
	static final int KEY_UP = 0x26;
	static final int KEY_RIGHT = 0x27;
	static final int KEY_DOWN = 0x28;

	static final int KEY_0 = 0x30;
	static final int KEY_1 = 0x31;
	static final int KEY_2 = 0x32;
	static final int KEY_3 = 0x33;
	static final int KEY_4 = 0x34;
	static final int KEY_5 = 0x35;
	static final int KEY_6 = 0x36;
	static final int KEY_7 = 0x37;
	static final int KEY_8 = 0x38;
	static final int KEY_9 = 0x39;

	static final int KEY_A = 0x41;
	static final int KEY_B = 0x42;
	static final int KEY_C = 0x43;
	static final int KEY_D = 0x44;
	static final int KEY_E = 0x45;
	static final int KEY_F = 0x46;
	static final int KEY_G = 0x47;
	static final int KEY_H = 0x48;
	static final int KEY_I = 0x49;
	static final int KEY_J = 0x4a;
	static final int KEY_K = 0x4b;
	static final int KEY_L = 0x4c;
	static final int KEY_M = 0x4d;
	static final int KEY_N = 0x4e;
	static final int KEY_O = 0x4f;
	static final int KEY_P = 0x50;
	static final int KEY_Q = 0x51;
	static final int KEY_R = 0x52;
	static final int KEY_S = 0x53;
	static final int KEY_T = 0x54;
	static final int KEY_U = 0x55;
	static final int KEY_V = 0x56;
	static final int KEY_W = 0x57;
	static final int KEY_X = 0x58;
	static final int KEY_Y = 0x59;
	static final int KEY_Z = 0x5a;

	protected static FFP ffp = new FFP();
	protected static FFPAUX aux = new FFPAUX(ffp);

	private static final int MIN_DIMENSION = 0x80;

	private static final int DEFAULT_REFRESH_RATE = 70;
	private static final int MINIMUM_SLEEP_TIME = 1;
	private static final int SECOND_MILLIS = 1000;

	static final int EUT_RGB = 0x00;
	static final int EUT_INDEX = 0x02;

	static final int EUT_NOT_RESIZABLE = 0x08;

	static final int EUT_LRES = 0x00;
	static final int EUT_HRES = 0x04;

	private int curr_fpoly_mode = 2;
	private int curr_bpoly_mode = 2;
	private int curr_fbpoly_mode = 2;
	private int curr_point_size = 1;

	private boolean update_graphics;

	private int fps;
	private int frame_time;

	private boolean req_resize = false;
	private int req_width = 0;
	private int req_height = 0;

	private boolean running = false;

	private int refresh_rate = DEFAULT_REFRESH_RATE;
	private int period = (int) ((1000f / refresh_rate) + 0.5f);

	private final boolean IS_RGB(int mask) {
		return (mask & EUT_INDEX) == 0;
	}

	private final boolean IS_INDEX(int mask) {
		return (mask & EUT_INDEX) != 0;
	}

	private final boolean IS_LRES(int mask) {
		return (mask & EUT_HRES) == 0;
	}

	private EUTCanvas canvas;
	private EUTFrame wind;

	protected int dbWidth;
	protected int dbHeight;

	private boolean closed;

	private BufferedImage image;
	protected int[] video;

	void init(int display_mode) {
		if (IS_LRES(display_mode)) {
			dbWidth = 512;
			dbHeight = 512;
		} else {
			dbWidth = 1024;
			dbHeight = 1024;
		}
		vid_init(dbWidth, dbHeight);
		wnd_create(dbWidth, dbHeight);
		System.out.println("Vendor: " + ffp.ffpGetString(FFP_VENDOR));
		System.out.println("Renderer: " + ffp.ffpGetString(FFP_RENDERER));
		System.out.println("Version: " + ffp.ffpGetString(FFP_VERSION));
		ffp.ffpVideoOutput(video, dbWidth, dbHeight);
		if (IS_INDEX(display_mode)) {
			ffp.ffpVideoPropertyi(FFP_VIDEO_COLOR_DITHER, FFP_TRUE);
		} else if (IS_RGB(display_mode)) {
			ffp.ffpVideoPropertyi(FFP_VIDEO_COLOR_DITHER, FFP_FALSE);
		}
		if ((display_mode & EUT_NOT_RESIZABLE) == EUT_NOT_RESIZABLE) {
			wind.setResizable(false);
		}
		start();
	}

	private void vid_init(int w, int h) {
		this.dbWidth = w;
		this.dbHeight = h;

		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice device = env.getDefaultScreenDevice();
		GraphicsConfiguration config = device.getDefaultConfiguration();
		image = null;
		video = null;
		image = config.createCompatibleImage(dbWidth, dbHeight);
		video = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
	}

	protected abstract void dbInit(int width, int height);

	protected abstract void dbUpdate();

	protected abstract void dbRender();

	private final void start() {
		Graphics g = canvas.getGraphics();
		dbInit(dbWidth, dbHeight);

		int frames = 0;
		long next_fps_upd = uclock() + SECOND_MILLIS;
		long time;

		long delay = MINIMUM_SLEEP_TIME;
		long last_time = uclock();

		int updates;

		running = true;
		try {
			keyReset();
			mouseReset();
			mouseResetActive();
			while (running) {
				long dt = last_time - uclock();

				if (delay > dt) {
					dt = delay;
				}

				updates = 0;
				sleep(dt);

				time = uclock();
				while (updates < 10 && (updates < 1 || (time > last_time))) {
					updates++;
					last_time += period;
				}

				if (time > last_time) {
					last_time = time;
				}

				if (req_resize) {
					int width = req_width;
					int height = req_height;
					if (width < MIN_DIMENSION) {
						width = MIN_DIMENSION;
					}
					if (height < MIN_DIMENSION) {
						height = MIN_DIMENSION;
					}
					this.dbWidth = width;
					this.dbHeight = height;

					req_resize = false;
					continue;
				}

				if (update_graphics) {
					g = canvas.getGraphics();
					update_graphics = false;
				}

				for (int i = 0; i < updates; i++) {
					update();
					keyReset();
					mouseReset();
				}

				time = uclock();

				long start_frame = uclock();

				render();

				g.drawImage(image, 0, 0, dbWidth, dbHeight, null);
				Toolkit.getDefaultToolkit().sync();

				frame_time += (uclock() - start_frame);
				frame_time >>= 1;

				frames++;

				if (time >= next_fps_upd) {
					fps = frames;
					frames = 0;
					next_fps_upd = time + SECOND_MILLIS;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} catch (OutOfMemoryError e) {
			e.printStackTrace();
		}
		terminate();
	}

	public final void dbStop() {
		running = false;
	}

	private final void terminate() {
		ffp.ffpDeleteAllTextures();

		sleep(SECOND_MILLIS);

		wnd_destroy();
	}

	private void update() {
		if (keyPressed(KEY_ESCAPE)) {
			dbStop();
		}
		if (keyPressed(KEY_7)) {
			curr_fpoly_mode++;
			curr_fpoly_mode %= 3;
			ffp.ffpPolygonMode(FFP_FRONT, FFP_POINT + curr_fpoly_mode);
		}
		if (keyPressed(KEY_8)) {
			curr_bpoly_mode++;
			curr_bpoly_mode %= 3;
			ffp.ffpPolygonMode(FFP_BACK, FFP_POINT + curr_bpoly_mode);
		}
		if (keyPressed(KEY_9)) {
			curr_fbpoly_mode++;
			curr_fbpoly_mode %= 3;
			ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_POINT + curr_fbpoly_mode);
		}
		if (keyPressed(KEY_0)) {
			curr_point_size++;
			curr_point_size %= 17;
			if (curr_point_size == 0) {
				curr_point_size = 1;
			}
			ffp.ffpPointSize(curr_point_size);
		}

		dbUpdate();
	}

	private void render() {
		dbRender();
	}

	private final long uclock() {
		return System.currentTimeMillis();
	}

	private final void sleep(long ms) {
		try {
			Thread.sleep(ms);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected final int getFPS() {
		return fps;
	}

	protected final int getFrameTime() {
		return frame_time;
	}
	
	static final float PI = 3.1415927f;

	protected final static float sin(double n) {
		return (float) Math.sin(n);
	}

	protected final static float cos(double n) {
		return (float) Math.cos(n);
	}

	protected final static float sqrt(double n) {
		return (float) Math.sqrt(n);
	}

	protected final static float pow(double a, double b) {
		return (float) Math.pow(a, b);
	}

	protected final static float random() {
		return (float) Math.random();
	}

	protected static float sign(double f) {
		return (float) Math.signum(f);
	}

	protected static float abs(double f) {
		return (float) Math.abs(f);
	}

	protected static int abs(int f) {
		return Math.abs(f);
	}

	protected static int min(int a, int b) {
		return Math.min(a, b);
	}

	protected static float atan(double f) {
		return (float) Math.atan(f);
	}

	protected static float log(double f) {
		return (float) Math.log(f);
	}

	protected static float hypot(double x, double y) {
		return (float) Math.hypot(x, y);
	}

	protected static float hypotsq(double x, double y) {
		return (float) (x * x + y * y);
	}

	protected static float exp(double x) {
		return (float) Math.exp(x);
	}

	private class EUTFrame extends Frame {

		private static final long serialVersionUID = -184153671122884691L;

		private EUTFrame() {
			super();
			enableEvents(AWTEvent.WINDOW_EVENT_MASK);
			enableEvents(AWTEvent.WINDOW_FOCUS_EVENT_MASK);
			enableEvents(AWTEvent.WINDOW_STATE_EVENT_MASK);
			enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
		}

		@Override
		public void processEvent(AWTEvent e) {
			process_event(e);
			super.processEvent(e);
		}

		void destroy() {
			disableEvents(AWTEvent.WINDOW_EVENT_MASK);
			disableEvents(AWTEvent.WINDOW_FOCUS_EVENT_MASK);
			disableEvents(AWTEvent.WINDOW_STATE_EVENT_MASK);
			disableEvents(AWTEvent.COMPONENT_EVENT_MASK);
			dispose();
		}

	}

	private void wnd_create(int width, int height) {
		canvas = new EUTCanvas();
		this.dbWidth = width;
		this.dbHeight = height;
		canvas.setSize(width, height);
		canvas.setMinimumSize(new Dimension(MIN_DIMENSION, MIN_DIMENSION));
		canvas.setIgnoreRepaint(true);
		wind = new EUTFrame();
		wind.setTitle("Envel");
		wind.setUndecorated(false);
		wind.setResizable(true);
		canvas.setFocusTraversalKeysEnabled(false);
		canvas.setFocusable(true);
		Point center = GraphicsEnvironment.getLocalGraphicsEnvironment().getCenterPoint();
		wind.setLocation(center.x - (width >> 1), center.y - (height >> 1));
		canvas.setVisible(true);
		wind.setVisible(true);
		wind.toFront();
		wind.add(canvas);
		wind.pack();
		canvas.requestFocus();
		canvas.requestFocusInWindow();
	}

	private void wnd_destroy() {
		if (!closed) {
			canvas.destroy();
			wind.destroy();
			closed = true;
		}
	}

	private static final AttributedCharacterIterator EMPTY_TEXT = (new AttributedString("")).getIterator();

	private class EUTCanvas extends Canvas implements InputMethodRequests {

		private static final long serialVersionUID = -4952262459876661617L;

		private final InputMethodRequests inputMethodRequests;

		private EUTCanvas() {
			super();
			if (System.getProperty("os.name").toLowerCase().startsWith("mac")) {
				inputMethodRequests = this;
			} else {
				inputMethodRequests = null;
				enableInputMethods(false);
			}
			enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
			enableEvents(AWTEvent.KEY_EVENT_MASK);
			enableEvents(AWTEvent.MOUSE_EVENT_MASK);
			enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
		}

		@Override
		public void processEvent(AWTEvent e) {
			process_event(e);
			super.processEvent(e);
		}

		private void destroy() {
			disableEvents(AWTEvent.COMPONENT_EVENT_MASK);
			disableEvents(AWTEvent.KEY_EVENT_MASK);
			disableEvents(AWTEvent.MOUSE_EVENT_MASK);
			disableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
		}

		@Override
		public InputMethodRequests getInputMethodRequests() {
			return inputMethodRequests;
		}

		@Override
		public Rectangle getTextLocation(TextHitInfo textHitInfo) {
			return new Rectangle(-0x8000, -0x8000, 0, 0);
		}

		@Override
		public TextHitInfo getLocationOffset(int x, int y) {
			return null;
		}

		@Override
		public int getInsertPositionOffset() {
			return 0;
		}

		@Override
		public AttributedCharacterIterator getCommittedText(int beginIndex, int endIndex, AttributedCharacterIterator.Attribute[] attributes) {
			return null;
		}

		@Override
		public int getCommittedTextLength() {
			return 0;
		}

		@Override
		public AttributedCharacterIterator cancelLatestCommittedText(AttributedCharacterIterator.Attribute[] attributes) {
			return null;
		}

		@Override
		public AttributedCharacterIterator getSelectedText(AttributedCharacterIterator.Attribute[] attributes) {
			return EMPTY_TEXT;
		}
	}

	private static final int MOUSE_BUTTON_NONE = 0;
	private static final int MOUSE_BUTTON_LEFT = 1;
	private static final int MOUSE_BUTTON_MIDDLE = 2;
	private static final int MOUSE_BUTTON_RIGHT = 4;

	private int pos_x = 0;
	private int pos_y = 0;
	private int active_buttons = 0;
	private int pressed_buttons = 0;

	private int[] pressed_keys = new int[0x100];
	private int[] held_keys = new int[0x100];

	private final void keyReset() {
		for (int i = 0; i < pressed_keys.length; i++) {
			pressed_keys[i] = 0x00;
		}
	}

	public final boolean keyPressed(int key) {
		return pressed_keys[key] == 0x01;
	}

	public final boolean keyHeld(int key) {
		return held_keys[key] == 0x01;
	}

	public final boolean keyPressed(char key) {
		int k = key - 32;
		if (k > 0 && k < pressed_keys.length) {
			return pressed_keys[k] == 0x01;
		}
		return false;
	}

	public final boolean keyHeld(char key) {
		int k = key - 32;
		if (k > 0 && k < held_keys.length) {
			return held_keys[k] == 0x01;
		}
		return false;
	}

	private final void mouseReset() {
		pressed_buttons = MOUSE_BUTTON_NONE;
	}

	private final void mouseResetActive() {
		active_buttons = MOUSE_BUTTON_NONE;
	}

	private final void mousePosition(int x, int y) {
		pos_x = x;
		pos_y = y;
	}

	private final int get_flag(int button) {
		switch (button) {
			case MouseEvent.BUTTON1:
				return MOUSE_BUTTON_LEFT;
			case MouseEvent.BUTTON2:
				return MOUSE_BUTTON_MIDDLE;
			case MouseEvent.BUTTON3:
				return MOUSE_BUTTON_RIGHT;
			default:
				return -1;
		}
	}

	public final int mouseX() {
		return pos_x;
	}

	public final int mouseY() {
		return pos_y;
	}

	public final boolean mouseLeftButtonActive() {
		return (active_buttons & MOUSE_BUTTON_LEFT) == MOUSE_BUTTON_LEFT;
	}

	public final boolean mouseMiddleButtonActive() {
		return (active_buttons & MOUSE_BUTTON_MIDDLE) == MOUSE_BUTTON_MIDDLE;
	}

	public final boolean mouseRightButtonActive() {
		return (active_buttons & MOUSE_BUTTON_RIGHT) == MOUSE_BUTTON_RIGHT;
	}

	public final boolean mouseLeftButtonPressed() {
		return (pressed_buttons & MOUSE_BUTTON_LEFT) == MOUSE_BUTTON_LEFT;
	}

	public final boolean mouseMiddleButtonPressed() {
		return (pressed_buttons & MOUSE_BUTTON_MIDDLE) == MOUSE_BUTTON_MIDDLE;
	}

	public final boolean mouseRightButtonPressed() {
		return (pressed_buttons & MOUSE_BUTTON_RIGHT) == MOUSE_BUTTON_RIGHT;
	}

	private void process_event(AWTEvent e) {
		if (e instanceof MouseEvent) {
			MouseEvent me = (MouseEvent) e;
			int eventX = me.getX();
			int eventY = me.getY();

			switch (e.getID()) {
				case MouseEvent.MOUSE_MOVED:
				case MouseEvent.MOUSE_DRAGGED:
					mousePosition(eventX, eventY);
					break;
				case MouseEvent.MOUSE_PRESSED:
					active_buttons |= get_flag(me.getButton());
					pressed_buttons |= get_flag(me.getButton());
					mousePosition(eventX, eventY);
					break;
				case MouseEvent.MOUSE_RELEASED:
					active_buttons &= ~get_flag(me.getButton());
					break;
				case MouseEvent.MOUSE_ENTERED:
					break;
				case MouseEvent.MOUSE_EXITED:
					break;
				default:
					break;
			}
			me.consume();
		} else if (e instanceof KeyEvent) {
			KeyEvent ke = (KeyEvent) e;
			int i = ke.getKeyCode();

			if (i < 0 || i > 0xff) {
				return;
			}

			switch (e.getID()) {
				case KEY_PRESSED:
					if (held_keys[i] == 0x00) {
						pressed_keys[i] = 0x01;
					}
					held_keys[i] = 0x01;
					break;
				case KEY_RELEASED:
					held_keys[i] = 0x00;
					break;
				default:
					break;
			}
			ke.consume();
		} else if (e instanceof WindowEvent) {
			switch (e.getID()) {
				case WindowEvent.WINDOW_CLOSING:
					dbStop();
					break;
				default:
					break;
			}
		} else if (e instanceof ComponentEvent) {
			update_graphics = true;
			switch (e.getID()) {
				case ComponentEvent.COMPONENT_RESIZED:
				case ComponentEvent.COMPONENT_MOVED:
					req_resize = true;
					req_width = canvas.getSize().width;
					req_height = canvas.getSize().height;
					break;
				default:
					break;
			}
		}
	}

}
