package ffp_examples;

public class accanti extends DemoBase implements jitter_h {

	public static void main(String[] args) {
		new accanti().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= h) {
			ffp.ffpOrtho(-2.25f, 2.25f, -2.25f * h / w, 2.25f * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-2.25f * w / h, 2.25f * w / h, -2.25f, 2.25f, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		float mat_ambient[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float light_position[] = { 0.0f, 0.0f, 10.0f, 1.0f };

		ffp.ffpMaterialfv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialfv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialf(FFP_SHININESS, 50.0f);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpShadeModel(FFP_FLAT);

		ffp.ffpClearColor(0.0f, 0.0f, 0.0f);
		ffp.ffpClearAccum(0.0f, 0.0f, 0.0f);
	}

	@Override
	public void dbUpdate() {

	}

	static final int ACSIZE = 8;

	@Override
	public void dbRender() {
		int[] viewport = new int[4];
		int jitter;

		ffp.ffpGetIntegerv(FFP_VIEWPORT, viewport);

		ffp.ffpClear(FFP_ACCUM_BUFFER);
		for (jitter = 0; jitter < ACSIZE; jitter++) {
			ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
			ffp.ffpPushMatrix();
			/*	Note that 4.5 is the distance in world space between
			 *	left and right and bottom and top.
			 *	This formula converts fractional pixel movement to 
			 *	world coordinates.
			 */
			ffp.ffpTranslatef(j8[jitter][0] * 4.5f / viewport[2], j8[jitter][1] * 4.5f / viewport[3], 0.0f);
			displayObjects();
			ffp.ffpPopMatrix();
			ffp.ffpFlush(FFP_MULTI_THREADED);
			ffp.ffpAccum(FFP_ACCUM, 1.0f / ACSIZE);
		}
		ffp.ffpFlush(FFP_MULTI_THREADED);
		ffp.ffpAccum(FFP_RETURN, 1.0f);
	}

	void displayObjects() {
		float torus_diffuse[] = { 0.7f, 0.7f, 0.0f };
		float cube_diffuse[] = { 0.0f, 0.7f, 0.7f };
		float sphere_diffuse[] = { 0.7f, 0.0f, 0.7f };
		float octa_diffuse[] = { 0.7f, 0.4f, 0.4f };

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(30.0f, 1.0f, 0.0f, 0.0f);

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.80f, 0.35f, 0.0f);
		ffp.ffpRotatef(100.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, torus_diffuse);
		aux.auxSolidTorus(0.275f, 0.85f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.75f, -0.50f, 0.0f);
		ffp.ffpRotatef(45.0f, 0.0f, 0.0f, 1.0f);
		ffp.ffpRotatef(45.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, cube_diffuse);
		aux.auxSolidCube(0.75f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.75f, 0.60f, 0.0f);
		ffp.ffpRotatef(30.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, sphere_diffuse);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.70f, -0.90f, 0.25f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, octa_diffuse);
		aux.auxSolidOctahedron();
		ffp.ffpPopMatrix();

		ffp.ffpPopMatrix();

	}

}
