package ffp_examples;

public class accpersp extends DemoBase implements jitter_h {

	public static void main(String[] args) {
		new accpersp().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
	}

	static final float PI_ = 3.1415927f;

	/**	accFrustum()
	 *  The first 6 arguments are identical to the ffp.ffpFrustum() call.
	 *  
	 *  pixdx and pixdy are anti-alias jitter in pixels. 
	 *  Set both equal to 0.0 for no anti-alias jitter.
	 *  eyedx and eyedy are depth-of field jitter in pixels. 
	 *  Set both equal to 0.0 for no depth of field effects.
	 *
	 *  focus is distance from eye to plane in focus. 
	 *  focus must be greater than, but not equal to 0.0.
	 *
	 *  Note that accFrustum() calls ffp.ffpTranslatef().  You will 
	 *  probably want to insure that your ModelView matrix has been 
	 *  initialized to identity before calling accFrustum().
	 */
	void accFrustum(float left, float right, float bottom, float top, float near, float far, float pixdx, float pixdy, float eyedx, float eyedy, float focus) {
		float xwsize, ywsize;
		float dx, dy;

		int[] viewport = new int[4];
		ffp.ffpGetIntegerv(FFP_VIEWPORT, viewport);

		xwsize = right - left;
		ywsize = top - bottom;

		dx = -(pixdx * xwsize / viewport[2] + eyedx * near / focus);
		dy = -(pixdy * ywsize / viewport[3] + eyedy * near / focus);

		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(left + dx, right + dx, bottom + dy, top + dy, near, far);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(-eyedx, -eyedy, 0.0f);
	}

	/**  accPerspective()
	 * 
	 *  The first 4 arguments are identical to the aux.ffpauxPerspective() call.
	 *  pixdx and pixdy are anti-alias jitter in pixels. 
	 *  Set both equal to 0.0 for no anti-alias jitter.
	 *  eyedx and eyedy are depth-of field jitter in pixels. 
	 *  Set both equal to 0.0 for no depth of field effects.
	 *
	 *  focus is distance from eye to plane in focus. 
	 *  focus must be greater than, but not equal to 0.0.
	 *
	 *  Note that accPerspective() calls accFrustum().
	 */
	void accPerspective(float fovy, float aspect, float near, float far, float pixdx, float pixdy, float eyedx, float eyedy, float focus) {
		float fov2, left, right, bottom, top;

		fov2 = ((fovy * PI_) / 180.0f) / 2.0f;

		top = near / (cos(fov2) / sin(fov2));
		bottom = -top;

		right = top * aspect;
		left = -right;

		accFrustum(left, right, bottom, top, near, far, pixdx, pixdy, eyedx, eyedy, focus);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		float mat_ambient[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float light_position[] = { 0.0f, 0.0f, 10.0f, 1.0f };

		ffp.ffpMaterialfv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialfv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialf(FFP_SHININESS, 50.0f);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpShadeModel(FFP_FLAT);

		ffp.ffpClearColor(0.0f, 0.0f, 0.0f);
		ffp.ffpClearAccum(0.0f, 0.0f, 0.0f);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_A)) {
			aa = !aa;
		}
	}

	static final int ACSIZE = 8;
	boolean aa = true;

	@Override
	public void dbRender() {
		int[] viewport = new int[4];

		ffp.ffpGetIntegerv(FFP_VIEWPORT, viewport);

		if (aa) {
			ffp.ffpClear(FFP_ACCUM_BUFFER);
			for (int jitter = 0; jitter < ACSIZE; jitter++) {
				ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
				accPerspective(50.0f, (float) viewport[2] / (float) viewport[3], 1.0f, 15.0f, j8[jitter][0], j8[jitter][1], 0.0f, 0.0f, 1.0f);

				displayObjects();
				ffp.ffpFlush(FFP_MULTI_THREADED);
				ffp.ffpAccum(FFP_ACCUM, 1.0f / ACSIZE);
			}
			ffp.ffpFlush(FFP_MULTI_THREADED);
			ffp.ffpAccum(FFP_RETURN, 1.0f);
			return;
		}

		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(50.0f, (float) viewport[2] / (float) viewport[3], 1.0f, 15.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();

		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		displayObjects();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	void displayObjects() {
		float torus_diffuse[] = { 0.7f, 0.7f, 0.0f };
		float cube_diffuse[] = { 0.0f, 0.7f, 0.7f };
		float sphere_diffuse[] = { 0.7f, 0.0f, 0.7f };
		float octa_diffuse[] = { 0.7f, 0.4f, 0.4f };

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.0f, 0.0f, -5.0f);
		ffp.ffpRotatef(30.0f, 1.0f, 0.0f, 0.0f);

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.80f, 0.35f, 0.0f);
		ffp.ffpRotatef(100.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, torus_diffuse);
		aux.auxSolidTorus(0.275f, 0.85f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.75f, -0.50f, 0.0f);
		ffp.ffpRotatef(45.0f, 0.0f, 0.0f, 1.0f);
		ffp.ffpRotatef(45.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, cube_diffuse);
		aux.auxSolidCube(0.75f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.75f, 0.60f, 0.0f);
		ffp.ffpRotatef(30.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, sphere_diffuse);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.70f, -0.90f, 0.25f);
		ffp.ffpMaterialfv(FFP_DIFFUSE, octa_diffuse);
		aux.auxSolidOctahedron();
		ffp.ffpPopMatrix();

		ffp.ffpPopMatrix();
	}

}
