package ffp_examples;

/*
 *  colormat.java
 *  After initialization, the program will be in
 *  ColorMaterial mode. Interaction: pressing the 
 *  mouse buttons will change the diffuse reflection values.
 */
public class colormat extends DemoBase {

	public static void main(String[] args) {
		new colormat().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= h) {
			ffp.ffpOrtho(-1.5f, 1.5f, -1.5f * h / w, 1.5f * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-1.5f * w / h, 1.5f * w / h, -1.5f, 1.5f, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	private static float diffuseMaterial[] = { 0.5f, 0.5f, 0.5f };
	private static int s = 25;

	/*
	 *  Initialize material property, light source, lighting model,
	 *  and depth buffer.
	 */
	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_DEPTH_TEST);

		float mat_specular[] = { 1, 1, 1 };
		float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };
		ffp.ffpMaterialfv(FFP_DIFFUSE, diffuseMaterial);
		ffp.ffpMaterialfv(FFP_SPECULAR, mat_specular);
		ffp.ffpMateriali(FFP_SHININESS, s);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpSmoothModel(FFP_PHONG);

		ffp.ffpColorMaterial(FFP_DIFFUSE);
		ffp.ffpEnable(FFP_COLOR_MATERIAL);
	}

	@Override
	public void dbUpdate() {
		if (mouseLeftButtonPressed()) {
			diffuseMaterial[0] += 0.1f;
			if (diffuseMaterial[0] > 1) {
				diffuseMaterial[0] = 0;
			}
			ffp.ffpColor3fv(diffuseMaterial);
		}
		if (mouseMiddleButtonPressed()) {
			diffuseMaterial[1] += 0.1f;
			if (diffuseMaterial[1] > 1) {
				diffuseMaterial[1] = 0;
			}
			ffp.ffpColor3fv(diffuseMaterial);
		}
		if (mouseRightButtonPressed()) {
			diffuseMaterial[2] += 0.1f;
			if (diffuseMaterial[2] > 1) {
				diffuseMaterial[2] = 0;
			}
			ffp.ffpColor3fv(diffuseMaterial);
		}
		if (keyPressed(KEY_W)) {
			s++;
			if (s > 128) {
				s = 0;
			}
			System.out.println(s);
			ffp.ffpMateriali(FFP_SHININESS, s);
		}
		if (keyPressed(KEY_S)) {
			s--;
			if (s < 0) {
				s = 128;
			}
			System.out.println(s);
			ffp.ffpMateriali(FFP_SHININESS, s);
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		aux.auxSolidSphere(1.0f, 20, 16);
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
