package ffp_examples;

/*
 * dither.java
 * This program demonstrates dithering in FFP.
 * A smooth shaded triangle is drawn and dithered
 * into the palette specified.
 */
public class dither extends DemoBase {

	public static void main(String[] args) {
		new dither().init(EUT_RGB | EUT_NOT_RESIZABLE);
	}

	static int r = 7, g = 9, b = 4;
	static boolean rotate = false;
	static float roty = 0;

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45.0f, (float) w / (float) h, 1.0f, 50.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -5);
	}

	@Override
	public void dbInit(int width, int height) {
		ffp.ffpViewport(0, 0, width, height);
		create_projection(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpVideoPropertyi(FFP_VIDEO_COLOR_DITHER, FFP_TRUE);
		ffp.ffpColormapCreate(r, g, b);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_W)) {
			rotate = !rotate;
		}
		if (keyPressed(KEY_A)) {
			roty = 0;
		}
		boolean change = false;
		if (keyPressed(KEY_R)) {
			r++;
			change = true;
		}
		if (keyPressed(KEY_G)) {
			g++;
			change = true;
		}
		if (keyPressed(KEY_B)) {
			b++;
			change = true;
		}
		r &= 0xff;
		g &= 0xff;
		b &= 0xff;
		if (r * g * b > 256) {
			r = 1;
			g = 1;
			b = 1;
		}
		if (change) {
			ffp.ffpColormapCreate(r, g, b);
		}
		if (keyPressed(KEY_R)) {
			r++;
			change = true;
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpRotatef(roty, 0, 1, 0);
		ffp.ffpBegin(FFP_TRIANGLES);
		ffp.ffpColor3ub(255, 0, 0);
		ffp.ffpVertex2f(-1.0f, -1.0f);
		ffp.ffpColor3ub(0, 255, 0);
		ffp.ffpVertex2f(1.0f, -1.0f);
		ffp.ffpColor3ub(0, 0, 255);
		ffp.ffpVertex2f(-1.0f, 1.0f);
		ffp.ffpEnd();
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
		if (rotate) {
			roty += 0.5f;
		}
	}

}
