package ffp_examples;

/*
 *  dmtex.java
 *  This program demonstrates double mapping by mapping
 *  two textures onto one rectangle.
 */
public class dmtex extends DemoBase {

	public static void main(String[] args) {
		new dmtex().init(EUT_RGB);
	}

	static int tex_width = 64;
	static int tex_height = 64;
	static int[] tex = new int[tex_width * tex_height];

	static int dmtex_width = 64;
	static int dmtex_height = 64;
	static int[] dmtex = new int[dmtex_width * dmtex_height];

	static void gen_textures() {
		for (int i = 0; i < tex_width; i++) {
			for (int j = 0; j < tex_height; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					tex[i + j * tex_width] = 0xffffff;
				} else {
					tex[i + j * tex_width] = 0;
				}
			}
		}
		for (int i = 0; i < dmtex_width; i++) {
			float ti = (float) (2.0 * PI * i / dmtex_width);
			for (int j = 0; j < dmtex_height; j++) {
				float tj = (float) (2.0 * PI * j / dmtex_height);
				dmtex[j + i * dmtex_width] = (int) (127 * (1.0 + sin(ti))) << 16 | (int) (127 * (1.0 + cos(2 * tj))) << 8 | (int) (127 * (1.0 + cos(ti + tj)));
			}
		}
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= h) {
			ffp.ffpOrtho(-0.5f, 1.5f, -0.5f * h / w, 1.5f * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-0.5f * w / h, 1.5f * w / h, -0.5f, 1.5f, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_FLAT);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_TEXTURE_2D);
		ffp.ffpEnable(FFP_DM_TEXTURE_2D);

		gen_textures();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
		ffp.ffpBindTexture(FFP_DM_TEXTURE_2D, 1);
		ffp.ffpTexParameteri(FFP_DM_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
		ffp.ffpTexImage2D(FFP_DM_TEXTURE_2D, FFP_RGB, dmtex, dmtex_width, dmtex_height);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_1)) {
			ffp.ffpTexParameteri(FFP_DM_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_ADD);
		}
		if (keyPressed(KEY_2)) {
			ffp.ffpTexParameteri(FFP_DM_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MODULATE);
		}
		if (keyPressed(KEY_3)) {
			ffp.ffpTexParameteri(FFP_DM_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_DECAL);

		}
		if (keyPressed(KEY_4)) {
			ffp.ffpTexParameteri(FFP_DM_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MIX);
		}
		if (keyPressed(KEY_T)) {
			boolean state = ffp.ffpIsEnabled(FFP_TEXTURE_2D);
			if (state) {
				ffp.ffpDisable(FFP_TEXTURE_2D);
			} else {
				ffp.ffpEnable(FFP_TEXTURE_2D);
			}
		}
		if (keyPressed(KEY_D)) {
			boolean state = ffp.ffpIsEnabled(FFP_DM_TEXTURE_2D);
			if (state) {
				ffp.ffpDisable(FFP_DM_TEXTURE_2D);
			} else {
				ffp.ffpEnable(FFP_DM_TEXTURE_2D);
			}
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		{
			ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
			ffp.ffpBindTexture(FFP_DM_TEXTURE_2D, 1);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpDMTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex2f(0.0f, 0.0f);
			ffp.ffpDMTexCoord2f(0.0f, 1f);
			ffp.ffpTexCoord2f(0.0f, 1f);
			ffp.ffpVertex2f(0.0f, 1.0f);
			ffp.ffpDMTexCoord2f(1f, 1f);
			ffp.ffpTexCoord2f(1f, 1f);
			ffp.ffpVertex2f(1.0f, 1.0f);
			ffp.ffpDMTexCoord2f(1f, 0.0f);
			ffp.ffpTexCoord2f(1f, 0.0f);
			ffp.ffpVertex2f(1.0f, 0.0f);
			ffp.ffpEnd();
		}
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
