package ffp_examples;

public class graph3d extends DemoBase {

	public static void main(String[] args) {
		new graph3d().init(EUT_RGB);
	}

	static final int N_FUNCTIONS = 20;

	static final int RES = 64;
	static final float DELTA = 2.0f / RES;
	static final float XN = RES * DELTA + 1;

	static float[][][] verts = new float[RES + 1][RES + 1][3];
	static float[][][] norms = new float[RES + 1][RES + 1][3];
	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;

	static float amp = 1f;
	static float t_freq = 1.0f;

	static float time;
	static int xold, yold;

	static int cfunc = 0;

	static float[] light_ambient = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float[] light_diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float[] light_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float[] light_position = { 1.0f, 2.5f, 1.0f, 1.0f };

	static boolean draw_normals = false;
	static boolean phong = false;

	static float mexicanhat(float x, float y, float t) {
		x /= 2f;
		y /= 2f;
		float tsq = hypot(x, y) * 4;
		tsq *= tsq;
		return (sin(t) - tsq) * exp(tsq / -2.0);
	}

	static float radialsin(float x, float y, float t) {
		x /= 0.15f;
		y /= 0.15f;
		return sin(hypot(x, y) - t) / 4.0f;
	}

	static float trigswap(float x, float y, float t) {
		x /= 0.15f;
		y /= 0.15f;
		return (cos(x) + sin(y) * cos(t)) / 2.0f;
	}

	static float sinsurface(float x, float y, float t) {
		x /= 0.15f;
		y /= 0.15f;
		return (pow(sin(x), 2) + pow(sin(y + t), 2)) / 4f;
	}

	static float torus(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return sqrt(0.16 - pow(0.6f - hypot(x, y), 2));
	}

	static float tube(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return 1.0f / (15 * (x * x + y * y));
	}

	static float landscape(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return sin(5 * x) * cos(5 * y) / 5;
	}

	static float cone(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return -hypot(x, y);
	}

	static float stairs(float x, float y, float t) {
		t *= t;
		y *= y;
		x /= 0.75f;
		return (sign(-.65f - x) + sign(-.35f - x) + sign(-.05f - x) + sign(.25f - x) + sign(.55f - x)) / 7;
	}

	static float pyramid(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (1 - abs(x + y) - abs(y - x)) * 0.75f;
	}

	static float paperplane(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (float) (sign(x) * atan(x * 80) / 6 * sign(-y - x + 1) * sign(-y + x + 1) * 5 - 1.01);
	}

	static float cross(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (float) (.1 - sign(sign(pow(x * 12, 2) - 9) - 1 + sign(pow(y * 12, 2) - 9) - 1) / 2);
	}

	static float dome(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (float) (.7 / log(hypotsq(x, y)) + .6);
	}

	static float roof(float x, float y, float t) {
		t *= t;
		x *= x;
		y /= 0.75f;
		return 1 - abs(y);
	}

	static float windmill(float x, float y, float t) {
		t *= t;
		x /= 5f;
		y /= 5f;
		return sign(x * y) * sign(1 - hypotsq(x * 9, y * 9)) / 9;
	}

	static float triangle(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (1 - sign(-x - .51f + abs(y * 2))) / 3 * (sign(.5f - x) + 1) / 3;
	}

	static float tophat(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (sign(0.2f - hypotsq(x, y)) + sign(0.2f - ((x * x) / 3 + (y * y) / 3))) / 3 - 1;
	}

	static float lettera(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return ((1 - sign(-x - .9f + abs(y * 2))) / 3 * (sign(.9f - x) + 1) / 3) * (sign(x + .65f) + 1) / 2 - ((1 - sign(-x - .39f + abs(y * 2))) / 3 * (sign(.9f - x) + 1) / 3) + ((1 - sign(-x - .39f + abs(y * 2))) / 3 * (sign(.6f - x) + 1) / 3) * (sign(x - .35f) + 1) / 2;
	}

	static float lettero(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (-sign(0.2f - hypotsq(x, y)) + sign(0.2f - ((x * x) / 3 + (y * y) / 3))) / 9;
	}

	static float letterv(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return sign(x - 1 + abs(y * 2)) / 3 + sign(x - .5f + abs(y * 2)) / 3;
	}

	static float fences(float x, float y, float t) {
		t *= t;
		x /= 0.75f;
		y /= 0.75f;
		return (float) (.75 / exp(pow(x * 5, 2) * pow(y * 5, 2)));
	}

	static float z(float x, float y, float t, int i) {
		float n = 0;
		switch (i) {
			case 0:
				n = mexicanhat(x, y, t * t_freq) * amp;
				break;
			case 1:
				n = radialsin(x, y, t * t_freq) * amp;
				break;
			case 2:
				n = trigswap(x, y, t * t_freq) * amp;
				break;
			case 3:
				n = sinsurface(x, y, t * t_freq) * amp;
				break;
			case 4:
				n = torus(x, y, t * t_freq) * amp;
				break;
			case 5:
				n = tube(x, y, t * t_freq) * amp;
				break;
			case 6:
				n = landscape(x, y, t * t_freq) * amp;
				break;
			case 7:
				n = cone(x, y, t * t_freq) * amp;
				break;
			case 8:
				n = stairs(x, y, t * t_freq) * amp;
				break;
			case 9:
				n = pyramid(x, y, t * t_freq) * amp;
				break;
			case 10:
				n = paperplane(x, y, t * t_freq) * amp;
				break;
			case 11:
				n = cross(x, y, t * t_freq) * amp;
				break;
			case 12:
				n = dome(x, y, t * t_freq) * amp;
				break;
			case 13:
				n = roof(x, y, t * t_freq) * amp;
				break;
			case 14:
				n = windmill(x, y, t * t_freq) * amp;
				break;
			case 15:
				n = triangle(x, y, t * t_freq) * amp;
				break;
			case 16:
				n = tophat(x, y, t * t_freq) * amp;
				break;
			case 17:
				n = lettera(x, y, t * t_freq) * amp;
				break;
			case 18:
				n = lettero(x, y, t * t_freq) * amp;
				break;
			case 19:
				n = letterv(x, y, t * t_freq) * amp;
				break;
			case 20:
				n = fences(x, y, t * t_freq) * amp;
				break;
			default:
				break;
		}
		if (!Float.isNaN(n) && Float.isFinite(n) && n <= 1 && n >= -1) {
			return n;
		}
		return -1;
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, w / (float) h, 0.1f, 150f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpMaterialfv(FFP_SPECULAR, light_specular);
		ffp.ffpMateriali(FFP_SHININESS, 127);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpLightModelfv(FFP_LIGHT_MODEL_AMBIENT, new float[] { 0.5f, 0.5f, 0.5f });
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_TRUE);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_TWO_SIDE, FFP_TRUE);
		ffp.ffpColorMaterial(FFP_DIFFUSE);
		ffp.ffpEnable(FFP_COLOR_MATERIAL);
	}

	@Override
	public void dbUpdate() {
		time += 0.01f;
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		if (keyPressed(KEY_N)) {
			draw_normals = !draw_normals;
		}

		if (keyPressed(KEY_S)) {
			phong = !phong;
			ffp.ffpSmoothModel(phong ? FFP_PHONG : FFP_GOURAUD);
		}
		if (keyPressed(KEY_R)) {
			t_freq += 0.1f;
		}
		if (keyPressed(KEY_F)) {
			t_freq -= 0.1f;
		}
		if (keyPressed(KEY_T)) {
			amp += 0.1f;
		}
		if (keyPressed(KEY_G)) {
			amp -= 0.1f;
		}
		if (keyPressed(KEY_1)) {
			cfunc++;
			cfunc %= N_FUNCTIONS;
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);

		for (int i = 0; i < RES; i++) {
			float x = i * DELTA - 1f;
			for (int j = 0; j < RES; j++) {
				float y = j * DELTA - 1f;
				verts[i][j][0] = x;
				verts[i][j][1] = z(x, y, time, cfunc);
				verts[i][j][2] = y;
			}
		}

		for (int i = 0; i < RES; i++) {
			for (int j = 0; j < RES; j++) {
				float v1x = verts[i][j + 1][0];
				float v1y = verts[i][j + 1][1];
				float v1z = verts[i][j + 1][2];

				float v2x = v1x;
				float v2y = verts[i][j][1];
				float v2z = verts[i][j][2];

				float v3x;
				float v3y;

				if (i < RES - 1) {
					v3x = verts[i + 1][j + 1][0];
					v3y = verts[i + 1][j + 1][1];
				} else {
					v3x = XN;
					v3y = z(XN, v1z, time, cfunc);
				}

				float v3z = v1z;

				float vax = v2x - v1x;
				float vay = v2y - v1y;
				float vaz = v2z - v1z;

				float vbx = v3x - v1x;
				float vby = v3y - v1y;
				float vbz = v3z - v1z;

				float nx = (vby * vaz) - (vbz * vay);
				float ny = (vbz * vax) - (vbx * vaz);
				float nz = (vbx * vay) - (vby * vax);

				float mag = sqrt(nx * nx + ny * ny + nz * nz);
				if (mag != 0) {
					mag = 1.0f / mag;
					norms[i][j + 1][0] = nx * mag;
					norms[i][j + 1][1] = ny * mag;
					norms[i][j + 1][2] = nz * mag;
				} else {
					norms[i][j + 1][0] = 0;
					norms[i][j + 1][1] = 1;
					norms[i][j + 1][2] = 0;
				}

			}
		}

		for (int i = 0; i < RES - 1; i++) {
			ffp.ffpBegin(FFP_TRIANGLE_STRIP);
			int next = i + 1;
			for (int j = RES - 1; j >= 0; j--) {
				ffp.ffpColor3fv(norms[i][j]);
				ffp.ffpNormal3fv(norms[i][j]);
				ffp.ffpVertex3fv(verts[i][j]);
				ffp.ffpColor3fv(norms[next][j]);
				ffp.ffpNormal3fv(norms[next][j]);
				ffp.ffpVertex3fv(verts[next][j]);
			}
			ffp.ffpEnd();
		}

		if (draw_normals) {
			ffp.ffpColor3f(1, 0, 0);
			ffp.ffpDisable(FFP_LIGHTING);
			ffp.ffpBegin(FFP_LINES);
			for (int i = 0; i < RES; i++) {
				for (int j = 0; j < RES; j++) {
					ffp.ffpVertex3fv(verts[i][j]);
					ffp.ffpVertex3f(verts[i][j][0] + norms[i][j][0] / 10, verts[i][j][1] + norms[i][j][1] / 10, verts[i][j][2] + norms[i][j][2] / 10);
				}
			}
			ffp.ffpEnd();
			ffp.ffpEnable(FFP_LIGHTING);
		}

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
