package ffp_examples;

/*
 *  polyhedra.java
 *  This program demonstrates the FFPAUX -hedron routines.
 *  Press F to toggle smooth shading, press W to render
 *  wireframes, left click to go to next polyhedra, and
 *  hold right click to rotate it.
 */
public class polyhedra extends DemoBase {

	static float light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };

	static int mat_ambient[] = { 9, 9, 9 };
	static int mat_specular[] = { 255, 255, 255 };
	static int mat_shininess[] = { 50 };
	static int model_ambient[] = { 9, 9, 9 };

	static float rot_y;
	static int curr_ph;
	static boolean wire;

	public static void main(String[] args) {
		new polyhedra().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 30.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -4);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpSmoothModel(FFP_PHONG);

		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, mat_shininess);
		ffp.ffpLightModeliv(FFP_LIGHT_MODEL_AMBIENT, model_ambient);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_F)) {
			boolean state = ffp.ffpGetInteger(FFP_SHADE_MODEL) == FFP_SMOOTH;
			if (state) {
				ffp.ffpShadeModel(FFP_FLAT);
			} else {
				ffp.ffpShadeModel(FFP_SMOOTH);
			}
		}
		if (mouseLeftButtonPressed()) {
			curr_ph = (curr_ph + 1) & 3;
		}
		if (mouseRightButtonActive()) {
			rot_y = (rot_y - 1) % 360;
		}
		if (keyPressed(KEY_W)) {
			wire = !wire;
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpRotatef(rot_y, 0.0f, 1.0f, 0.0f);
		switch (curr_ph) {
			case 0:
				if (wire) {
					aux.auxWireTetrahedron();
				} else {
					aux.auxSolidTetrahedron();
				}
				break;
			case 1:
				if (wire) {
					aux.auxWireIcosahedron();
				} else {
					aux.auxSolidIcosahedron();
				}
				break;
			case 2:
				if (wire) {
					aux.auxWireOctahedron();
				} else {
					aux.auxSolidOctahedron();
				}
				break;
			case 3:
				if (wire) {
					aux.auxWireDodecahedron();
				} else {
					aux.auxSolidDodecahedron();
				}
				break;
			default:
				break;
		}
		ffp.ffpPopMatrix();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
