package ffp_examples;

/*
 * prim.java
 * This program demonstrates the primitives in FFP.
 */
public class prim extends DemoBase {

	public static void main(String[] args) {
		new prim().init(EUT_RGB);
	}

	static final int PIXEL_CENTER(float x) {
		return (int) (x + 0.5f);
	}

	static final int GAP = 10;
	static final int ROWS = 3;
	static final int COLS = 3;

	static boolean mode1 = true, mode2 = true;
	static int windW, windH;
	static int boxW, boxH;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= h) {
			aux.auxOrtho2D(-1.0f, 1.0f, -1.0f, 1.0f * h / w);
		} else {
			aux.auxOrtho2D(-1.0f, 1.0f * w / h, -1.0f, 1.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		windW = width;
		windH = height;
	}

	static void set_color(int index) {
		switch (index) {
			case 0:
				ffp.ffpColor3ub(0, 0, 0);
				break;
			case 1:
				ffp.ffpColor3ub(0xff, 0, 0);
				break;
			case 2:
				ffp.ffpColor3ub(0, 0xff, 0);
				break;
			case 3:
				ffp.ffpColor3ub(0xff, 0xff, 0);
				break;
			case 4:
				ffp.ffpColor3ub(0, 0, 0xff);
				break;
			case 5:
				ffp.ffpColor3ub(0xff, 0, 0xff);
				break;
			case 6:
				ffp.ffpColor3ub(0, 0xff, 0xff);
				break;
			case 7:
				ffp.ffpColor3ub(0xff, 0xff, 0xff);
				break;
			default:
				break;
		}

	}

	static void set_viewport(int row, int column) {
		boxW = (windW - (COLS + 1) * GAP) / COLS;
		boxH = (windH - (ROWS + 1) * GAP) / ROWS;

		int x = GAP + column * (boxW + GAP);
		int y = GAP + row * (boxH + GAP);

		ffp.ffpViewport(x, y, boxW, boxH);

		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpOrtho(-boxW / 2, boxW / 2, -boxH / 2, boxH / 2, 0.0f, 1.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	static void ex_points() {
		ffp.ffpBegin(FFP_POINTS);
		for (int i = 1; i < 8; i++) {
			int j = i * 2;
			set_color(i);
			ffp.ffpVertex2i(-j, -j);
			ffp.ffpVertex2i(-j, 0);
			ffp.ffpVertex2i(-j, j);
			ffp.ffpVertex2i(0, j);
			ffp.ffpVertex2i(j, j);
			ffp.ffpVertex2i(j, 0);
			ffp.ffpVertex2i(j, -j);
			ffp.ffpVertex2i(0, -j);
		}
		ffp.ffpEnd();
	}

	static void ex_lines() {
		ffp.ffpPushMatrix();

		ffp.ffpTranslatef(-12, 0, 0);
		for (int i = 1; i < 8; i++) {
			set_color(i);
			ffp.ffpBegin(FFP_LINES);
			ffp.ffpVertex2i(-boxW / 4, -boxH / 4);
			ffp.ffpVertex2i(boxW / 4, boxH / 4);
			ffp.ffpEnd();
			ffp.ffpTranslatef(4, 0, 0);
		}

		ffp.ffpPopMatrix();
	}

	static void ex_line_strip() {
		ffp.ffpBegin(FFP_LINE_STRIP);
		set_color(1);
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 4), PIXEL_CENTER(-boxH / 4));
		set_color(2);
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 4), PIXEL_CENTER(boxH / 4));
		set_color(3);
		ffp.ffpVertex2f(PIXEL_CENTER(boxW / 4), PIXEL_CENTER(boxH / 4));
		set_color(4);
		ffp.ffpVertex2f(PIXEL_CENTER(boxW / 4), PIXEL_CENTER(-boxH / 4));
		ffp.ffpEnd();
	}

	static void ex_line_loop() {
		ffp.ffpBegin(FFP_LINE_LOOP);
		set_color(1);
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 4), PIXEL_CENTER(-boxH / 4));
		set_color(2);
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 4), PIXEL_CENTER(boxH / 4));
		set_color(3);
		ffp.ffpVertex2f(PIXEL_CENTER(boxW / 4), PIXEL_CENTER(boxH / 4));
		set_color(4);
		ffp.ffpVertex2f(PIXEL_CENTER(boxW / 4), PIXEL_CENTER(-boxH / 4));
		ffp.ffpEnd();

		set_color(5);
		ffp.ffpBegin(FFP_LINE_LOOP);
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 8), PIXEL_CENTER(-boxH / 8));
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 8), PIXEL_CENTER(boxH / 8));
		ffp.ffpEnd();
		ffp.ffpBegin(FFP_LINE_LOOP);
		ffp.ffpVertex2f(PIXEL_CENTER(-boxW / 8), PIXEL_CENTER(boxH / 8 + 5));
		ffp.ffpVertex2f(PIXEL_CENTER(boxW / 8), PIXEL_CENTER(boxH / 8 + 5));
		ffp.ffpEnd();

		set_color(6);
		ffp.ffpBegin(FFP_POINTS);
		ffp.ffpVertex2i(0, 0);
		ffp.ffpEnd();
	}

	static void ex_triangles() {
		ffp.ffpBegin(FFP_TRIANGLES);
		set_color(1);
		ffp.ffpVertex2i(-boxW / 4, -boxH / 4);
		set_color(2);
		ffp.ffpVertex2i(-boxW / 8, -boxH / 16);
		set_color(3);
		ffp.ffpVertex2i(boxW / 8, -boxH / 16);

		set_color(4);
		ffp.ffpVertex2i(-boxW / 4, boxH / 4);
		set_color(5);
		ffp.ffpVertex2i(-boxW / 8, boxH / 16);
		set_color(6);
		ffp.ffpVertex2i(boxW / 8, boxH / 16);
		ffp.ffpEnd();
	}

	static void ex_triangle_strip() {
		ffp.ffpBegin(FFP_TRIANGLE_STRIP);
		set_color(1);
		ffp.ffpVertex2i(-boxW / 4, -boxH / 4);
		set_color(2);
		ffp.ffpVertex2i(-boxW / 4, boxH / 4);
		set_color(3);
		ffp.ffpVertex2i(0, -boxH / 4);
		set_color(4);
		ffp.ffpVertex2i(0, boxH / 4);
		set_color(5);
		ffp.ffpVertex2i(boxW / 4, -boxH / 4);
		set_color(6);
		ffp.ffpVertex2i(boxW / 4, boxH / 4);
		ffp.ffpEnd();
	}

	static void ex_triangle_fan() {
		int[][] vx = new int[8][2];
		int x0, y0, x1, y1, x2, y2, x3, y3;
		int i;

		y0 = -boxH / 4;
		y1 = y0 + boxH / 2 / 3;
		y2 = y1 + boxH / 2 / 3;
		y3 = boxH / 4;
		x0 = -boxW / 4;
		x1 = x0 + boxW / 2 / 3;
		x2 = x1 + boxW / 2 / 3;
		x3 = boxW / 4;

		vx[0][0] = x0;
		vx[0][1] = y1;
		vx[1][0] = x0;
		vx[1][1] = y2;
		vx[2][0] = x1;
		vx[2][1] = y3;
		vx[3][0] = x2;
		vx[3][1] = y3;
		vx[4][0] = x3;
		vx[4][1] = y2;
		vx[5][0] = x3;
		vx[5][1] = y1;
		vx[6][0] = x2;
		vx[6][1] = y0;
		vx[7][0] = x1;
		vx[7][1] = y0;

		ffp.ffpBegin(FFP_TRIANGLE_FAN);
		set_color(7);
		ffp.ffpVertex2i(0, 0);
		for (i = 0; i < 8; i++) {
			set_color(7 - i);
			ffp.ffpVertex2iv(vx[i]);
		}
		ffp.ffpEnd();
	}

	static void ex_rect() {
		ffp.ffpColor3f(0xff, 0, 0xff);
		ffp.ffpRecti(-boxW / 4, -boxH / 4, boxW / 4, boxH / 4);
	}

	static void ex_quads() {
		ffp.ffpBegin(FFP_QUADS);
		set_color(1);
		ffp.ffpVertex2i(-boxW / 4, -boxH / 4);
		set_color(2);
		ffp.ffpVertex2i(-boxW / 8, -boxH / 16);
		set_color(3);
		ffp.ffpVertex2i(boxW / 8, -boxH / 16);
		set_color(4);
		ffp.ffpVertex2i(boxW / 4, -boxH / 4);

		set_color(5);
		ffp.ffpVertex2i(-boxW / 4, boxH / 4);
		set_color(6);
		ffp.ffpVertex2i(-boxW / 8, boxH / 16);
		set_color(7);
		ffp.ffpVertex2i(boxW / 8, boxH / 16);
		set_color(4);
		ffp.ffpVertex2i(boxW / 4, boxH / 4);
		ffp.ffpEnd();

		ffp.ffpColor3f(0xff, 0xff, 0xff);
	}

	static void ex_quad_strip() {
		ffp.ffpBegin(FFP_QUAD_STRIP);
		set_color(1);
		ffp.ffpVertex2i(-boxW / 4, -boxH / 4);
		set_color(2);
		ffp.ffpVertex2i(-boxW / 4, boxH / 4);
		set_color(3);
		ffp.ffpVertex2i(0, -boxH / 4);
		set_color(4);
		ffp.ffpVertex2i(0, boxH / 4);
		set_color(5);
		ffp.ffpVertex2i(boxW / 4, -boxH / 4);
		set_color(6);
		ffp.ffpVertex2i(boxW / 4, boxH / 4);
		ffp.ffpEnd();
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_1)) {
			mode1 = !mode1;
		}
		if (keyPressed(KEY_2)) {
			mode2 = !mode2;
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpViewport(0, 0, windW, windH);

		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpClear(FFP_VIDEO_BUFFER);

		if (mode1) {
			ffp.ffpShadeModel(FFP_SMOOTH);
		} else {
			ffp.ffpShadeModel(FFP_FLAT);
		}

		if (mode2) {
			ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_FILL);
		} else {
			ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_LINE);
		}

		set_viewport(0, 0);
		ex_lines();
		set_viewport(0, 1);
		ex_line_strip();
		set_viewport(0, 2);
		ex_line_loop();

		set_viewport(1, 0);
		ex_triangle_fan();
		set_viewport(1, 1);
		ex_triangles();
		set_viewport(1, 2);
		ex_triangle_strip();

		set_viewport(2, 0);
		ex_quads();
		set_viewport(2, 1);
		ex_quad_strip();
		set_viewport(2, 2);
		ex_points();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
