package ffp_examples;

/*
 *  quadric.java
 *  This program demonstrates the use of some of the FFPAUX Quadric
 *  routines. Note that the cylinder has no top or bottom and the disk
 *  has a hole in it.
 */
public class quadric extends DemoBase {

	static int mat_ambient[] = { 55, 55, 55 };
	static int mat_specular[] = { 255, 255, 255 };
	static int mat_shininess[] = { 50 };
	static float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };
	static int model_ambient[] = { 55, 55, 55 };

	public static void main(String[] args) {
		new quadric().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		float v = 2.5f;
		if (w <= h) {
			ffp.ffpOrtho(-v, v, -v * h / w, v * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-v * w / h, v * w / h, -v, v, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);

		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, mat_shininess);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpLightModeliv(FFP_LIGHT_MODEL_AMBIENT, model_ambient);
	}

	@Override
	public void dbUpdate() {

	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpTranslatef(-1.0f, -1.0f, 0.0f);
		aux.auxSphere(0.75f, 15, 10, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_SMOOTH, FFPAUX_FALSE);

		ffp.ffpShadeModel(FFP_FLAT);
		ffp.ffpTranslatef(0.0f, 2.0f, 0.0f);
		ffp.ffpPushMatrix();
		ffp.ffpRotatef(300.0f, 1.0f, 0.0f, 0.0f);
		aux.auxCylinder(0.5f, 0.3f, 1.0f, 15, 5, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_FLAT, FFPAUX_FALSE);
		ffp.ffpPopMatrix();

		ffp.ffpDisable(FFP_LIGHTING);
		ffp.ffpColor3ub(0, 255, 255);
		ffp.ffpTranslatef(2.0f, -2.0f, 0.0f);
		aux.auxDisk(0.25f, 1.0f, 20, 4, FFPAUX_LINE, FFPAUX_OUTSIDE, FFPAUX_NONE, FFPAUX_FALSE);

		ffp.ffpColor3ub(255, 255, 0);
		ffp.ffpTranslatef(0.0f, 2.0f, 0.0f);
		aux.auxPartialDisk(0.0f, 1.0f, 20, 4, 0.0f, 225.0f, FFPAUX_SILHOUETTE, FFPAUX_OUTSIDE, FFPAUX_NONE, FFPAUX_FALSE);

		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
