package ffp_examples;

/*
 *  render2tex.java
 *  Rendering to a texture
 */
public class render2tex extends DemoBase {

	public static void main(String[] args) {
		new render2tex().init(EUT_RGB);
	}

	static final int SIZE = 512;

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 10;
	static int xold, yold;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(20.0f, (float) w / (float) h, 5.0f, 15.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	private static void cubeProj() {
		/* Define a view-port adapted to the texture */
		ffp.ffpViewport(0, 0, SIZE, SIZE);
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(20, 1, 5, 15);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_DEPTH_TEST);

		int[] texture = new int[SIZE * SIZE];
		for (int i = 0; i < texture.length; i++) {
			texture[i] = 0xffffff;
		}
		ffp.ffpEnable(FFP_TEXTURE_2D);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, texture, SIZE, SIZE);
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}

		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		// Render to buffer 
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);

		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		cubeProj();

		aux.auxSolidCube(1);

		ffp.ffpFlush(FFP_MULTI_THREADED);

		// Copy buffer to texture 
		ffp.ffpCopyTexSubImage2D(FFP_TEXTURE_2D, 5, 5, 0, 0, SIZE - 10, SIZE - 10);
		create_projection(dbWidth, dbHeight);

		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		aux.auxSolidCube(1);

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}
	
}
