package ffp_examples;

public class rotate extends DemoBase {

	public static void main(String[] args) {
		new rotate().init(EUT_RGB | EUT_NOT_RESIZABLE);
	}

	static int dmtex_width = 64;
	static int dmtex_height = 64;
	static int[] dmtex = new int[dmtex_width * dmtex_height];
	static float rot = 0;
	static float xt = 0;
	static float yt = 0;

	static void gen_textures() {
		for (int i = 0; i < dmtex_width; i++) {
			float ti = (float) (2.0 * PI * i / dmtex_width);
			for (int j = 0; j < dmtex_height; j++) {
				float tj = (float) (2.0 * PI * j / dmtex_height);
				dmtex[j + i * dmtex_width] = (int) (127 * (1.0 + sin(ti))) << 16 | (int) (127 * (1.0 + cos(2 * tj))) << 8 | (int) (127 * (1.0 + cos(ti + tj)));
			}
		}
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		float v = 1f;
		if (w <= h) {
			ffp.ffpOrtho(0.0f, v, 0.0f, v * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(0.0f, v * w / h, 0.0f, v, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_FLAT);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_TEXTURE_2D);

		gen_textures();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, dmtex, dmtex_width, dmtex_height);
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			xt = (float) x / dbWidth;
			yt = (float) -y / dbHeight;
		}
		if (keyPressed(KEY_T)) {
			boolean state = ffp.ffpIsEnabled(FFP_TEXTURE_2D);
			if (state) {
				ffp.ffpDisable(FFP_TEXTURE_2D);
			} else {
				ffp.ffpEnable(FFP_TEXTURE_2D);
			}
		}

		rot += 0.2f;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(xt - 0.25f, yt + 0.75f, 0);
		{
			ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
			ffp.ffpScalef(0.5f, 0.5f, 0.5f);
			ffp.ffpTranslatef(0.5f, 0.5f, 0);
			ffp.ffpRotatef(rot, 0, 0, 1);
			ffp.ffpTranslatef(-0.5f, -0.5f, 0);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex2f(0.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 1f);
			ffp.ffpVertex2f(0.0f, 1.0f);
			ffp.ffpTexCoord2f(1f, 1f);
			ffp.ffpVertex2f(1.0f, 1.0f);
			ffp.ffpTexCoord2f(1f, 0.0f);
			ffp.ffpVertex2f(1.0f, 0.0f);
			ffp.ffpEnd();
		}
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
