package ffp_examples;

import ffp.FFPAUX;

public class shadow extends DemoBase {

	public static void main(String[] args) {
		new shadow().init(EUT_RGB);
	}

	int winxsize;
	int winysize;
	int texxsize = 128;
	int texysize = 128;

	int floorTexture = 0;
	int shadowTextures = 1;

	float[] lightposORIG = new float[] { 0, 0, -350, 1 };
	float[] lightpos = new float[] { 0, 0, -350, 1 };

	int numShadowTex;

	/* list of polygons that have shadow textures */
	float pts[][][] = {
			/* floor */
			{ { -100.f, -100.f, -320.f }, { -100.f, -100.f, -520.f }, { 100.f, -100.f, -320.f }, { 100.f, -100.f, -520.f } },

			/* left wall */
			{ { -100.f, -100.f, -320.f }, { -100.f, 100.f, -320.f }, { -100.f, -100.f, -520.f }, { -100.f, 100.f, -520.f } },

			/* back wall */
			{ { -100.f, -100.f, -520.f }, { -100.f, 100.f, -520.f }, { 100.f, -100.f, -520.f }, { 100.f, 100.f, -520.f } },

			/* right wall */
			{ { 100.f, -100.f, -520.f }, { 100.f, 100.f, -520.f }, { 100.f, -100.f, -320.f }, { 100.f, 100.f, -320.f } },

			/* ceiling */
			{ { -100.f, 100.f, -520.f }, { -100.f, 100.f, -320.f }, { 100.f, 100.f, -520.f }, { 100.f, 100.f, -320.f } },

			/* blue panel */
			{ { -60.f, -40.f, -400.f }, { -60.f, 70.f, -400.f }, { -30.f, -40.f, -480.f }, { -30.f, 70.f, -480.f } },

			/* yellow panel */
			{ { -40.f, -50.f, -400.f }, { -40.f, 50.f, -400.f }, { -10.f, -50.f, -450.f }, { -10.f, 50.f, -450.f } },

			/* red panel */
			{ { -20.f, -60.f, -400.f }, { -20.f, 30.f, -400.f }, { 10.f, -60.f, -420.f }, { 10.f, 30.f, -420.f } },

			/* green panel */
			{ { 0.f, -70.f, -400.f }, { 0.f, 10.f, -400.f }, { 30.f, -70.f, -395.f }, { 30.f, 10.f, -395.f }

			} };

	float materials[][] = { { 1.0f, 1.0f, 1.0f, 1.0f }, /* floor        */
			{ 1.0f, 1.0f, 1.0f, 1.0f }, /* left wall    */
			{ 1.0f, 1.0f, 1.0f, 1.0f }, /* back wall    */
			{ 1.0f, 1.0f, 1.0f, 1.0f }, /* right wall   */
			{ 1.0f, 1.0f, 1.0f, 1.0f }, /* ceiling      */
			{ 0.2f, 0.5f, 1.0f, 1.0f }, /* blue panel   */
			{ 1.0f, 0.6f, 0.0f, 1.0f }, /* yellow panel */
			{ 1.0f, 0.2f, 0.2f, 1.0f }, /* red panel    */
			{ 0.3f, 0.9f, 0.6f, 1.0f }, /* green panel  */
	};

	/* some simple vector utility routines */

	void vcopy(float a[], float b[]) {
		b[0] = a[0];
		b[1] = a[1];
		b[2] = a[2];
	}

	void vnormalize(float v[]) {
		float m = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
		v[0] /= m;
		v[1] /= m;
		v[2] /= m;
	}

	void vadd(float a[], float b[], float c[]) {
		c[0] = a[0] + b[0];
		c[1] = a[1] + b[1];
		c[2] = a[2] + b[2];
	}

	void vsub(float a[], float b[], float c[]) {
		c[0] = a[0] - b[0];
		c[1] = a[1] - b[1];
		c[2] = a[2] - b[2];
	}

	void vcross(float a[], float b[], float c[]) {
		c[0] = a[1] * b[2] - a[2] * b[1];
		c[1] = -(a[0] * b[2] - a[2] * b[0]);
		c[2] = a[0] * b[1] - a[1] * b[0];
	}

	float vdot(float a[], float b[]) {
		return (a[0] * b[0] + a[1] * b[1] + a[2] * b[2]);
	}

	void findNormal(float pts[][], float normal[]) {
		FFPAUX.auxFaceNormal333fv(pts[2], pts[1], pts[0], normal);
	}

	static float origin[] = { 0.f, 0.f, 0.f, 1.f };
	static float black[] = { 0.f, 0.f, 0.f, 1.f };
	static float ambient[] = { 0.2f, 0.2f, 0.2f, 1.f };

	void make_shadow_texture(int index, float[] eyept, float dx, float dy) {
		float[] xaxis = new float[3], yaxis = new float[3], zaxis = new float[3];
		float[] cov = new float[3]; /* center of view */
		float[] pte = new float[3]; /* plane to eye */
		float[] eye = new float[3];
		float[] tmp = new float[3], normal = new float[3];
		float dist;
		float[][] qpts = pts[index];
		float left, right, bottom, top;
		float znear = 10.f, zfar = 600.f;

		/* For simplicity, we don't compute the transformation matrix described */
		/* in Heckbert and Herf's paper.  The transformation and frustum used   */
		/* here is much simpler.                                                */

		vcopy(eyept, eye);
		vsub(qpts[1], qpts[0], yaxis);
		vsub(qpts[2], qpts[0], xaxis);
		vcross(yaxis, xaxis, zaxis);

		vnormalize(zaxis);
		vnormalize(xaxis); /* x-axis of eye coord system, in object space */
		vnormalize(yaxis); /* y-axis of eye coord system, in object space */

		/* jitter the eyepoint */
		eye[0] += xaxis[0] * dx;
		eye[1] += xaxis[1] * dx;
		eye[2] += xaxis[2] * dx;
		eye[0] += yaxis[0] * dy;
		eye[1] += yaxis[1] * dy;
		eye[2] += yaxis[2] * dy;

		/* center of view is just eyepoint offset in direction of normal */
		vadd(eye, zaxis, cov);

		/* set up viewing matrix */
		ffp.ffpPushMatrix();
		ffp.ffpLoadIdentity();
		aux.auxLookAt(eye[0], eye[1], eye[2], cov[0], cov[1], cov[2], yaxis[0], yaxis[1], yaxis[2]);

		/* compute a frustum that just encloses the polygon */
		vsub(qpts[0], eye, tmp); /* from eye to 0th vertex */
		left = vdot(tmp, xaxis);
		vsub(qpts[2], eye, tmp); /* from eye to 2nd vertex */
		right = vdot(tmp, xaxis);
		vsub(qpts[0], eye, tmp); /* from eye to 0th vertex */
		bottom = vdot(tmp, yaxis);
		vsub(qpts[1], eye, tmp); /* from eye to 1st vertex */
		top = vdot(tmp, yaxis);

		/* scale the frustum values based on the distance to the polygon */
		vsub(qpts[0], eye, pte);
		dist = abs(vdot(zaxis, pte));
		left *= (znear / dist);
		right *= (znear / dist);
		bottom *= (znear / dist);
		top *= (znear / dist);

		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpPushMatrix();
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(left, right, bottom, top, znear, zfar);
		ffp.ffpMatrixMode(FFP_MODELVIEW);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materials[index]);

		ffp.ffpDisable(FFP_TEXTURE_2D);

		for (int n = 0; n < numShadowTex; n++) {
			qpts = pts[n];

			ffp.ffpColor1i(255);
			findNormal(qpts, normal);
			ffp.ffpNormal3fv(normal);
			if (n == index) {
				/* draw this poly with ambient and diffuse lighting */
				ffp.ffpEnableLight(0);
			} else {
				/* draw other polys with ambient lighting only */
				ffp.ffpDisableLight(0);
			}
			ffp.ffpBegin(FFP_TRIANGLE_STRIP);
			ffp.ffpVertex3fv(qpts[0]);
			ffp.ffpVertex3fv(qpts[1]);
			ffp.ffpVertex3fv(qpts[2]);
			ffp.ffpVertex3fv(qpts[3]);
			ffp.ffpEnd();
		}
		ffp.ffpPopMatrix();
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpPopMatrix();
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	void make_all_shadow_textures(float[] eye, float dx, float dy) {
		int texPerRow = (winxsize / texxsize);
		for (int n = 0; n < numShadowTex; n++) {
			int x = (n % texPerRow) * texxsize;
			int y = (n / texPerRow) * texysize;
			ffp.ffpViewport(x, y, texxsize, texysize);
			make_shadow_texture(n, eye, dx, dy);
		}
		ffp.ffpViewport(0, 0, winxsize, winysize);
	}

	void store_all_shadow_textures() {
		int texPerRow = (winxsize / texxsize);
		for (int n = 0; n < numShadowTex; n++) {
			int x = (n % texPerRow) * texxsize;
			int y = (n / texPerRow) * texysize;
			ffp.ffpBindTexture(FFP_TEXTURE_2D, shadowTextures + n);
			ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
			ffp.ffpCopyTexImage2D(FFP_TEXTURE_2D, x, y, texxsize, texysize);
		}

	}

	static final int NOSHADOWS = 0;
	static final int SOFTSHADOWS = 1;
	static final int HARDSHADOWS = 2;

	int shadowMode = HARDSHADOWS;
	boolean viewTextures = false;

	int[] make_texture(int w, int h) {
		int[] texture = new int[w * h];

		for (int i = 0; i < w; i++) {
			for (int j = 0; j < h; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					texture[i + j * w] = 0xffffff;
				} else {
					texture[i + j * w] = 0;
				}
			}
		}
		return texture;
	}

	private static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(-30, 30, -30, 30, 100, 640);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		create_projection();
		winxsize = width;
		winysize = height;

		ffp.ffpSmoothModel(FFP_PHONG);

		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpLightfv(0, FFP_AMBIENT, black);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_TRUE);
		// number of shadow textures to make
		numShadowTex = pts.length;
		int[] tex = make_texture(texxsize, texysize);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, floorTexture);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, texxsize, texysize);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_V)) {
			viewTextures = !viewTextures;
		}
		if (keyPressed(KEY_S)) {
			shadowMode++;
			shadowMode %= 3;
		}
		r += 0.04f;
		s += 0.04f;
		t += 0.04f;
	}

	float r, s, t;

	@Override
	public void dbRender() {
		float[] normal = new float[3];
		float[][] qpts;
		lightpos[0] = 0.2f * lightposORIG[0] + sin(r) * 48;
		lightpos[1] = 0.1f * lightposORIG[1] + cos(s) * 32;
		lightpos[2] = lightposORIG[2] + sin(t) * 16;

		ffp.ffpPushMatrix();
		ffp.ffpLoadIdentity();
		if (shadowMode != NOSHADOWS) {
			ffp.ffpLightfv(0, FFP_POSITION, origin);
		} else {
			ffp.ffpLightfv(0, FFP_POSITION, lightpos);
		}
		ffp.ffpPopMatrix();

		if (shadowMode == SOFTSHADOWS) {
			float jitterSize;
			float dx, dy;
			int numSteps, i, j;

			/* size of the area to jitter the light in */
			jitterSize = 15.0f;

			/* number of times along x and y to jitter */
			numSteps = 5;

			ffp.ffpClear(FFP_ACCUM_BUFFER);
			for (j = 0; j < numSteps; j++) {
				for (i = 0; i < numSteps; i++) {

					/* compute jitter amount, centering the jitter steps around zero */
					dx = (i - (numSteps - 1.0f) / 2.0f) / (numSteps - 1.0f) * jitterSize;
					dy = (j - (numSteps - 1.0f) / 2.0f) / (numSteps - 1.0f) * jitterSize;

					ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
					make_all_shadow_textures(lightpos, dx, dy);
					ffp.ffpFlush(FFP_MULTI_THREADED);
					ffp.ffpAccum(FFP_ACCUM, 1.0f / (numSteps * numSteps));
				}
			}
			ffp.ffpAccum(FFP_RETURN, 1.0f);
			store_all_shadow_textures();

		} else if (shadowMode == HARDSHADOWS) {
			/* make shadow textures from just one frame */
			ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
			make_all_shadow_textures(lightpos, 0, 0);
			ffp.ffpFlush(FFP_MULTI_THREADED);
			store_all_shadow_textures();
		}
		if (viewTextures) {
			ffp.ffpFlush(FFP_MULTI_THREADED);
			return;
		}

		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		create_projection();
		ffp.ffpColor1i(255);
		if (shadowMode == NOSHADOWS) {
			ffp.ffpEnable(FFP_LIGHTING);
			ffp.ffpEnableLight(0);
			ffp.ffpDisable(FFP_TEXTURE_2D);
			ffp.ffpBindTexture(FFP_DM_TEXTURE_2D, floorTexture);
			for (int i = 0; i < numShadowTex; i++) {
				ffp.ffpState(FFP_DM_TEXTURE_2D, i == 0);
				qpts = pts[i];

				findNormal(qpts, normal);
				ffp.ffpNormal3fv(normal);
				ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materials[i]);
				ffp.ffpBegin(FFP_TRIANGLE_STRIP);
				ffp.ffpDMTexCoord2f(0, 0);
				ffp.ffpVertex3fv(qpts[0]);
				ffp.ffpDMTexCoord2f(0, 1);
				ffp.ffpVertex3fv(qpts[1]);
				ffp.ffpDMTexCoord2f(1, 0);
				ffp.ffpVertex3fv(qpts[2]);
				ffp.ffpDMTexCoord2f(1, 1);
				ffp.ffpVertex3fv(qpts[3]);
				ffp.ffpEnd();
			}
		} else {
			ffp.ffpDisable(FFP_LIGHTING);
			ffp.ffpEnable(FFP_DM_TEXTURE_2D);

			ffp.ffpEnable(FFP_TEXTURE_2D);
			ffp.ffpBindTexture(FFP_DM_TEXTURE_2D, floorTexture);

			for (int i = 0; i < numShadowTex; i++) {
				ffp.ffpState(FFP_DM_TEXTURE_2D, i == 0);
				qpts = pts[i];
				ffp.ffpBindTexture(FFP_TEXTURE_2D, shadowTextures + i);
				ffp.ffpBegin(FFP_TRIANGLE_STRIP);
				ffp.ffpTexCoord2f(0, 0);
				ffp.ffpDMTexCoord2f(0, 0);
				ffp.ffpVertex3fv(qpts[0]);
				ffp.ffpTexCoord2f(0, 1);
				ffp.ffpDMTexCoord2f(0, 1);
				ffp.ffpVertex3fv(qpts[1]);
				ffp.ffpTexCoord2f(1, 0);
				ffp.ffpDMTexCoord2f(1, 0);
				ffp.ffpVertex3fv(qpts[2]);
				ffp.ffpTexCoord2f(1, 1);
				ffp.ffpDMTexCoord2f(1, 1);
				ffp.ffpVertex3fv(qpts[3]);
				ffp.ffpEnd();
			}
		}
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(lightpos[0], lightpos[1], lightpos[2]);
		ffp.ffpColor3ub(255, 0, 0);
		aux.auxWireCube(10);
		ffp.ffpColor1i(255);
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
