package ffp_examples;

/*
 *  spheremap.java
 *  This program demonstrates sphere mapping
 *  on a torus with double mapping. Left click
 *  and right click to rotate the torus.
 */
public class spheremap extends DemoBase {

	public static void main(String[] args) {
		new spheremap().init(EUT_RGB);
	}

	static float light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	/* light_position is NOT default value */
	static float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };

	static int tex_width = 64;
	static int tex_height = 64;
	static int[] tex = new int[tex_width * tex_height];

	static int dmtex_width = 64;
	static int dmtex_height = 64;
	static int[] dmtex = new int[dmtex_width * dmtex_height];

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;
	static int xold, yold;

	static void gen_textures() {
		for (int i = 0; i < tex_width; i++) {
			for (int j = 0; j < tex_height; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					tex[i + j * tex_width] = 0xffffff;
				} else {
					tex[i + j * tex_width] = 0;
				}
			}
		}
		for (int i = 0; i < dmtex_width; i++) {
			float ti = (float) (2.0 * PI * i / dmtex_width);
			for (int j = 0; j < dmtex_height; j++) {
				float tj = (float) (2.0 * PI * j / dmtex_height);
				dmtex[j + i * dmtex_width] = (int) (127 * (1.0 + sin(ti))) << 16 | (int) (127 * (1.0 + cos(2 * tj))) << 8 | (int) (127 * (1.0 + cos(ti + tj)));
			}
		}
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 300.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpSmoothModel(FFP_PHONG);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_TEXTURE_2D);
		ffp.ffpEnable(FFP_DM_TEXTURE_2D);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);

		gen_textures();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MODULATE);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_SPHERE_MAP, FFP_TRUE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 1);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MIX);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, dmtex, dmtex_width, dmtex_height);
		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpMaterialfv(FFP_DIFFUSE, light_diffuse);
		ffp.ffpMaterialfv(FFP_SPECULAR, light_specular);
		ffp.ffpMaterialf(FFP_SHININESS, 100);
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		{
			ffp.ffpLoadIdentity();
			ffp.ffpTranslatef(0, 0, -translate_z);
			ffp.ffpRotatef(rotate_y, 1, 0, 0);
			ffp.ffpRotatef(rotate_x, 0, 1, 0);
			ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
			ffp.ffpBindTexture(FFP_DM_TEXTURE_2D, 1);
			aux.auxSolidTorus(0.275f, 0.85f, 64, 64);
		}
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
