package ffp_examples;

/*
 * spot.java
 * This program demonstrates spotlights.
 */
public class spot extends DemoBase {

	static final float M_PI = 3.14159265f;
	static final float TWO_PI = 2 * M_PI;

	static class Light {
		float[] diff;
		float[] pos;
		float[] spotDir;
		float spotExp;
		float spotCutoff;
		float[] atten;
		float[] trans;
		float[] rot;
		float[] swing;
		float[] arc;
		float[] arcIncr;
	}

	static float[] modelAmb = { 0.2f, 0.2f, 0.2f };
	static float[] matAmb = { 0.2f, 0.2f, 0.2f };
	static float[] matDiff = { 0.8f, 0.8f, 0.8f };
	static float[] matSpec = { 1, 1, 1 };
	static int xold, yold;

	static Light[] spots = new Light[3];

	public static void main(String[] args) {
		new spot().init(EUT_RGB);
	}

	private static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, 1, 0.1f, 10);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -2);
		ffp.ffpRotatef(45, 1, 0, 0);
	}

	@Override
	public void dbInit(int width, int height) {
		create_projection();
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpSmoothModel(FFP_PHONG);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnable(FFP_NORMALIZE);
		ffp.ffpLightModelfv(FFP_LIGHT_MODEL_AMBIENT, modelAmb);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_TRUE);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_TWO_SIDE, FFP_FALSE);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_RGB, FFP_TRUE);

		ffp.ffpMaterialfv(FFP_AMBIENT, matAmb);
		ffp.ffpMaterialfv(FFP_DIFFUSE, matDiff);
		ffp.ffpMaterialfv(FFP_SPECULAR, matSpec);
		ffp.ffpMaterialf(FFP_SHININESS, 100.0f);

		spots[0] = new Light();
		Light l = spots[0];
		l.diff = new float[] { 0.8f, 0.0f, 0.0f }; /* diffuse */
		l.pos = new float[] { 0.0f, 0.0f, 0.0f, 1.0f }; /* position */
		l.spotDir = new float[] { 0.0f, -1.0f, 0.0f }; /* direction */
		l.spotExp = 10.0f;
		l.spotCutoff = 10.0f; /* exponent, cutoff */
		l.atten = new float[] { 1.0f, 0.0f, 0.0f }; /* attenuation */
		l.trans = new float[] { 0.0f, 0.75f, 0.0f }; /* translation */
		l.rot = new float[] { 0.0f, 0.0f, 0.0f }; /* rotation */
		l.swing = new float[] { -20.0f, 0.0f, 40.0f }; /* swing */
		l.arc = new float[] { 0.0f, 0.0f, 0.0f }; /* arc */
		l.arcIncr = new float[] { TWO_PI / 280.0f, 0.0f, TWO_PI / 560.0f }; /* arc increment */

		spots[1] = new Light();
		l = spots[1];
		l.diff = new float[] { 0.0f, 0.8f, 0.0f }; /* diffuse */
		l.pos = new float[] { 0.0f, 0.0f, 0.0f, 1.0f }; /* position */
		l.spotDir = new float[] { 0.0f, -1.0f, 0.0f }; /* direction */
		l.spotExp = 20.0f;
		l.spotCutoff = 30.0f; /* exponent, cutoff */
		l.atten = new float[] { 1.0f, 0.0f, 0.0f }; /* attenuation */
		l.trans = new float[] { 0.0f, 0.75f, 0.0f }; /* translation */
		l.rot = new float[] { 0.0f, 0.0f, 0.0f }; /* rotation */
		l.swing = new float[] { -20.0f, 0.0f, -40.0f }; /* swing */
		l.arc = new float[] { 0.0f, 0.0f, 0.0f }; /* arc */
		l.arcIncr = new float[] { TWO_PI / 480.0f, 0.0f, TWO_PI / 240.0f }; /* arc increment */

		spots[2] = new Light();
		l = spots[2];
		l.diff = new float[] { 0.0f, 0.0f, 0.8f }; /* diffuse */
		l.pos = new float[] { 0.0f, 0.0f, 0.0f, 1.0f }; /* position */
		l.spotDir = new float[] { 0.0f, -1.0f, 0.0f }; /* direction */
		l.spotExp = 20.0f;
		l.spotCutoff = 20.0f; /* exponent, cutoff */
		l.atten = new float[] { 1.0f, 0.0f, 0.0f }; /* attenuation */
		l.trans = new float[] { 0.0f, 0.75f, 0.0f }; /* translation */
		l.rot = new float[] { 0.0f, 0.0f, 0.0f }; /* rotation */
		l.swing = new float[] { 20.0f, 0.0f, 40.0f }; /* swing */
		l.arc = new float[] { 0.0f, 0.0f, 0.0f }; /* arc */
		l.arcIncr = new float[] { TWO_PI / 200.0f, 0.0f, TWO_PI / 400.0f }; /* arc increment */

		initLights();
	}

	static void initLights() {
		for (int k = 0; k < spots.length; ++k) {
			int lt = k;
			Light s = spots[k];

			ffp.ffpEnableLight(lt);
			ffp.ffpLightfv(lt, FFP_DIFFUSE, s.diff);
			ffp.ffpLightfv(lt, FFP_SPECULAR, matSpec);

			ffp.ffpLightf(lt, FFP_SPOT_EXPONENT, s.spotExp);
			ffp.ffpLightf(lt, FFP_SPOT_CUTOFF, s.spotCutoff);
			ffp.ffpLightf(lt, FFP_CONSTANT_ATTENUATION, s.atten[0]);
			ffp.ffpLightf(lt, FFP_LINEAR_ATTENUATION, s.atten[1]);
			ffp.ffpLightf(lt, FFP_QUADRATIC_ATTENUATION, s.atten[2]);
		}
	}

	static void aimLights() {
		for (int k = 0; k < spots.length; ++k) {
			Light s = spots[k];

			s.rot[0] = s.swing[0] * sin(s.arc[0]);
			s.arc[0] += s.arcIncr[0];
			if (s.arc[0] > TWO_PI)
				s.arc[0] -= TWO_PI;

			s.rot[1] = s.swing[1] * sin(s.arc[1]);
			s.arc[1] += s.arcIncr[1];
			if (s.arc[1] > TWO_PI)
				s.arc[1] -= TWO_PI;

			s.rot[2] = s.swing[2] * sin(s.arc[2]);
			s.arc[2] += s.arcIncr[2];
			if (s.arc[2] > TWO_PI)
				s.arc[2] -= TWO_PI;
		}
	}

	static void setLights() {
		for (int k = 0; k < spots.length; ++k) {
			int lt = k;
			Light s = spots[k];

			ffp.ffpPushMatrix();
			ffp.ffpTranslatef(s.trans[0], s.trans[1], s.trans[2]);
			ffp.ffpRotatef(s.rot[0], 1, 0, 0);
			ffp.ffpRotatef(s.rot[1], 0, 1, 0);
			ffp.ffpRotatef(s.rot[2], 0, 0, 1);
			ffp.ffpLightfv(lt, FFP_POSITION, s.pos);
			ffp.ffpLightfv(lt, FFP_SPOT_DIRECTION, s.spotDir);
			ffp.ffpPopMatrix();
		}
	}

	static void drawLights() {
		ffp.ffpDisable(FFP_LIGHTING);
		for (int k = 0; k < spots.length; ++k) {
			Light s = spots[k];

			ffp.ffpColor3fv(s.diff);

			ffp.ffpPushMatrix();
			ffp.ffpTranslatef(s.trans[0], s.trans[1], s.trans[2]);
			ffp.ffpRotatef(s.rot[0], 1, 0, 0);
			ffp.ffpRotatef(s.rot[1], 0, 1, 0);
			ffp.ffpRotatef(s.rot[2], 0, 0, 1);
			ffp.ffpBegin(FFP_LINES);
			ffp.ffpVertex3f(s.pos[0], s.pos[1], s.pos[2]);
			ffp.ffpVertex3f(s.spotDir[0], s.spotDir[1], s.spotDir[2]);
			ffp.ffpEnd();
			ffp.ffpPopMatrix();
		}
		ffp.ffpEnable(FFP_LIGHTING);
	}

	static void drawPlane(int w, int h) {
		float dw = 1.0f / w;
		float dh = 1.0f / h;

		ffp.ffpNormal3f(0.0f, 0.0f, 1.0f);
		for (int j = 0; j < h; j++) {
			ffp.ffpBegin(FFP_TRIANGLE_STRIP);
			for (int i = 0; i <= w; i++) {
				ffp.ffpVertex2f(dw * i, dh * (j + 1));
				ffp.ffpVertex2f(dw * i, dh * j);
			}
			ffp.ffpEnd();
		}
	}

	float spin = 0;

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			spin = spin + (x - xold) / 5.0f;
			if (spin > 90) {
				spin = 90;
			}
			if (spin < -90) {
				spin = -90;
			}
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(spin, 0, 1, 0);

		aimLights();
		setLights();

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(-90.0f, 1, 0, 0);
		ffp.ffpTranslatef(-0.5f, -0.5f, 0.0f);
		drawPlane(1, 1);
		ffp.ffpScalef(0.1f, 0.1f, 0.1f);
		ffp.ffpTranslatef(0, 5, 1);
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(5f, 0, 0);
		aux.auxSolidTorus(1, 1, 16, 16);
		ffp.ffpPopMatrix();
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(2.5f, 0, 0);
		aux.auxSolidDodecahedron();
		ffp.ffpPopMatrix();
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(7.5f, 0, 0);
		aux.auxSolidCube(1);
		ffp.ffpPopMatrix();
		ffp.ffpPopMatrix();

		drawLights();
		ffp.ffpPopMatrix();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
