package ffp_examples;

/*
 *  zbuf.java
 *  This program demonstrates the use of the FFP lighting model.
 *  Objects are drawn using a grey material characteristic. 
 *  A single light source illuminates the objects.
 */
public class zbuf extends DemoBase {

	static float light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	/* light_position is NOT default value */
	static float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };

	private boolean zbuffer = true;

	public static void main(String[] args) {
		new zbuf().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, (float) w / h, 1, 100);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);

		ffp.ffpMaterialfv(FFP_SPECULAR, light_specular);

		ffp.ffpEnable(FFP_CULL_FACE);
		ffp.ffpEnable(FFP_COLOR_MATERIAL);
		ffp.ffpColorMaterial(FFP_DIFFUSE);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_Z)) {
			zbuffer = !zbuffer;
		}
	}

	@Override
	public void dbRender() {
		if (zbuffer) {
			ffp.ffpEnable(FFP_DEPTH_TEST);
		} else {
			ffp.ffpDisable(FFP_DEPTH_TEST);
		}
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0, 0, -5);
		ffp.ffpRotatef(20.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpRotatef(-30.0f, 0.0f, 1.0f, 0.0f);

		ffp.ffpPushMatrix();
		ffp.ffpColor3ub(192, 0, 0);
		aux.auxSolidCube(1);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0f, 0.5f, 1.0f);
		ffp.ffpRotatef(90.0f, 0.0f, 1.0f, 0.0f);
		ffp.ffpColor3ub(0, 192, 0);
		aux.auxSolidTorus(0.25f, 0.75f, 15, 15);
		ffp.ffpPopMatrix();

		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
